@@ -33,6 +33,7 @@ using System.Diagnostics;
 namespace Beagle.Util {
 
 	public enum LogLevel {
+		Always,
 		Error,
 		Warn,
 		Debug,
@@ -93,11 +94,22 @@ namespace Beagle.Util {
 				console_redirect_writer = console_log_writer;
 			}
 
+			// Redirect the console output to a special file
 			Console.SetOut (console_redirect_writer);
 			Console.SetError (console_redirect_writer);
 
-			// If we are running in the background, redirect stdin to /dev/null
 			if (! running_in_foreground) {
+				// Now redirect the *actual* stdout/stderr to our main
+				// log file.  This is used to catch Mono crash reports
+				// in our logs.  Note that this doesn't override the
+				// console redirection above, which is good.
+				StreamWriter sw = (StreamWriter) log_writer;
+				FileStream fs = (FileStream) sw.BaseStream;
+				int fd = (int) fs.Handle;
+				Mono.Unix.Native.Syscall.dup2 (fd, 1); // stdout
+				Mono.Unix.Native.Syscall.dup2 (fd, 2); // stderr
+
+				// If we are running in the background, redirect stdin to /dev/null
 				FileStream dev_null_stream = new FileStream ("/dev/null",
 									     FileMode.Open,
 									     FileAccess.Read,
@@ -199,7 +211,7 @@ namespace Beagle.Util {
 
 		static private void WriteLine (LogLevel level, string format, object [] args, Exception ex) 
 		{
-			if (cutoff_level < level)
+			if (level != LogLevel.Always && cutoff_level < level)
 				return;
 
 			string ex_str = null;
@@ -241,6 +253,9 @@ namespace Beagle.Util {
 			case LogLevel.Debug:
 				prefix_builder.Append ("DEBUG");
 				break;
+			case LogLevel.Always:
+				prefix_builder.Append (" INFO");
+				break;
 			default:
 				prefix_builder.Append (" HUH?");
 				break;
@@ -338,6 +353,21 @@ namespace Beagle.Util {
 		{
 			WriteLine (LogLevel.Error, null, null, ex);
 		}
+
+		static public void Always (string message, params object [] args)
+		{
+			WriteLine (LogLevel.Always, message, args, null);
+		}
+		
+		static public void Always (Exception ex, string message, params object [] args)
+		{
+			WriteLine (LogLevel.Always, message, args, ex);
+		}
+
+		static public void Always (Exception ex)
+		{
+			WriteLine (LogLevel.Always, null, null, ex);
+		}
 	}
 }
 
