@@ -265,6 +265,14 @@ namespace Beagle.Daemon
 				Environment.Exit (0);
 			}
 
+			// Setup some exclude patterns akin to FSQ/FileNameFilter.cs
+			denied_patterns.Add (new ExcludeItem (ExcludeType.Pattern, "*~"));
+			denied_patterns.Add (new ExcludeItem (ExcludeType.Pattern, "#*#"));
+			denied_patterns.Add (new ExcludeItem (ExcludeType.Pattern, "*.o"));
+			denied_patterns.Add (new ExcludeItem (ExcludeType.Pattern, "*.a"));
+			denied_patterns.Add (new ExcludeItem (ExcludeType.Pattern, "*.aux"));
+			denied_patterns.Add (new ExcludeItem (ExcludeType.Pattern, "*.tmp"));
+
 			Log.Always ("Starting beagle-build-index (pid {0}) at {1}", Process.GetCurrentProcess ().Id, DateTime.Now);
 
 			// Set system priorities so we don't slow down the system
@@ -449,6 +457,17 @@ namespace Beagle.Daemon
 				FlushIndexer (driver, request);
 				// FlushIndexer clears the pending_request
 			}
+
+			
+			// Super Lame Hack: gtk-sharp up to 2.10 requires a main loop
+			// to dispose of any managed wrappers around GObjects.  Since
+			// we don't have one, we'll process all the pending items in
+			// a loop here.  This is particularly an issue with maildirs,
+			// because we need the loop to clean up after GMime.  Without
+			// it, GMime's streams are never completely unref'd, the
+			// file descriptors aren't closed, and we run out and crash.
+			while (GLib.MainContext.Pending ())
+				GLib.MainContext.Iteration ();
 		}
 
 		static IndexerReceipt [] FlushIndexer (IIndexer indexer, IndexerRequest request)
@@ -463,6 +482,8 @@ namespace Beagle.Daemon
 			ArrayList pending_children;
 			pending_children = new ArrayList ();
 
+			fa_store.BeginTransaction ();
+
 			foreach (IndexerReceipt raw_r in receipts) {
 
 				if (raw_r is IndexerAddedReceipt) {
@@ -504,6 +525,8 @@ namespace Beagle.Daemon
 				}
 			}
 
+			fa_store.CommitTransaction ();
+
 			request.Clear (); // clear out the old request
 
 			if (Shutdown.ShutdownRequested)
