@@ -134,19 +134,6 @@ namespace Beagle.Daemon {
 		static ArrayList queryables = new ArrayList ();
 		static Hashtable iqueryable_to_queryable = new Hashtable ();
 
-		static bool ThisApiSoVeryIsBroken (Type m, object criteria)
-		{
-			return m == (Type) criteria;
-		}
-
-		static bool TypeImplementsInterface (Type t, Type iface)
-		{
-			Type[] impls = t.FindInterfaces (new TypeFilter (ThisApiSoVeryIsBroken),
-							 iface);
-			return impls.Length > 0;
-		}
-
-		// Find the types in the assembly that
 		// (1) register themselves in AssemblyInfo.cs:IQueryableTypes and
 		// (2) has a QueryableFlavor attribute attached
 		// assemble a Queryable object and stick it into our list of queryables.
@@ -329,6 +316,8 @@ namespace Beagle.Daemon {
 			assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_BACKEND_PATH", PathFinder.BackendDir);
 		}
 
+		private static bool queryables_started = false;
+
 		static public void Start ()
 		{
 			// Only add the executing assembly if we haven't already loaded it.
@@ -367,6 +356,8 @@ namespace Beagle.Daemon {
 				q.Start ();
 			}
 
+			queryables_started = true;
+
 			return false;
 		}
 
@@ -621,6 +612,12 @@ namespace Beagle.Daemon {
 
 		static public bool IsIndexing {
 			get {
+				// If the backends haven't been started yet,
+				// there is at least the initial setup.  Just
+				// assume all the backends are indexing.
+				if (! queryables_started)
+					return true;
+
 				foreach (Queryable q in queryables) {
 					QueryableStatus status = q.GetQueryableStatus ();
 
@@ -633,6 +630,6 @@ namespace Beagle.Daemon {
 
 				return false;
 			}
-		}					
+		}
 	}
 }
