@@ -104,7 +104,7 @@ namespace Search.Tiles {
 						       Gtk.SelectionData selectionData,
 						       uint info, uint time)
 		{
-			byte[] data = System.Text.Encoding.UTF8.GetBytes (Hit.Uri + "\r\n");
+			byte[] data = System.Text.Encoding.UTF8.GetBytes (Hit.EscapedUri + "\r\n");
 			selectionData.Set (selectionData.Target, 8, data);
 		}
 
@@ -181,7 +181,9 @@ namespace Search.Tiles {
 
 #if ENABLE_OPEN_WITH
 			if (EnableOpenWith) {
-				OpenWithMenu owm = new OpenWithMenu (Hit ["beagle:MimeType"]);
+				// FIXME: Not sure if going with the parent is
+				// the right thing to do in all cases.
+				OpenWithMenu owm = new OpenWithMenu (Utils.GetFirstPropertyOfParent (hit, "beagle:MimeType"));
 				owm.ApplicationActivated += OpenWith;
 				owm.AppendToMenu (menu);
 			}
@@ -322,7 +324,7 @@ namespace Search.Tiles {
 		private void OpenWith (Gnome.Vfs.MimeApplication mime_application)
 		{
 			GLib.List uri_list = new GLib.List (typeof (string));
-			uri_list.Append (Hit.UriAsString);
+			uri_list.Append (Hit.EscapedUri);
 			mime_application.Launch (uri_list);
 		}
 #endif
@@ -340,7 +342,10 @@ namespace Search.Tiles {
 #if ENABLE_DESKTOP_LAUNCH
 			command = "desktop-launch";
 			expects_uris = true;
-#else		       
+#elif ENABLE_XDG_OPEN
+			command = "xdg-open";
+			expects_uris = true;
+#else
 			GnomeFu.VFSMimeApplication app;
 			app = GnomeFu.GetDefaultAction (hit.MimeType);
 			if (app.command != null) {
@@ -352,10 +357,17 @@ namespace Search.Tiles {
 				Console.WriteLine ("Can't open MimeType '{0}'", hit.MimeType);
 				return;
 			}
-
-			if (expects_uris)
-				item = UriFu.UriToSerializableString (hit.Uri);
-			else
+			
+			if (expects_uris) {
+				// FIXME: I'm not sure that opening the parent
+				// URI (if present) is the right thing to do in
+				// all cases, but it does work for all our
+				// current cases.
+				if (hit.ParentUri != null)
+					item = hit.EscapedParentUri;
+				else
+					item = hit.EscapedUri;
+			} else
 				item = hit.Path;
 
 			// Sometimes the command is 'quoted'
@@ -404,14 +416,19 @@ namespace Search.Tiles {
 
 		public void OpenFromUri (Uri uri)
 		{
-			OpenFromUri (uri.ToString ());
+			OpenFromUri (UriFu.UriToEscapedString (uri));
 		}
 
 		public void OpenFromUri (string uri)
                 {
-#if ENABLE_DESKTOP_LAUNCH
+#if ENABLE_DESKTOP_LAUNCH || ENABLE_XDG_OPEN
 			SafeProcess p = new SafeProcess ();
+
+#  if ENABLE_DESKTOP_LAUNCH
 			p.Arguments = new string[] { "desktop-launch", uri };
+#  elif ENABLE_XDG_OPEN
+			p.Arguments = new string[] { "xdg-open", uri };
+#  endif
 
 			try {
 				p.Start ();
