@@ -1,6 +1,7 @@
 using System;
 using System.Globalization;
 using Mono.Unix;
+using Beagle.Util;
 
 namespace Search.Tiles {
 
@@ -21,15 +22,15 @@ namespace Search.Tiles {
 
 		public static TileGroupInfo[] GroupInfo = new TileGroupInfo[] {
 			new TileGroupInfo (TileGroup.Application,
-					   Catalog.GetString ("Application"), 1),
+					   Catalog.GetString ("Applications"), 1),
 			new TileGroupInfo (TileGroup.Contact,
-					   Catalog.GetString ("Contact"), 2),
+					   Catalog.GetString ("Contacts"), 2),
 			new TileGroupInfo (TileGroup.Calendar,
 					   Catalog.GetString ("Calendar Events"), 2),
 			new TileGroupInfo (TileGroup.Folder,
-					   Catalog.GetString ("Folder"), 2),
+					   Catalog.GetString ("Folders"), 2),
 			new TileGroupInfo (TileGroup.Image,
-					   Catalog.GetString ("Image"), 2),
+					   Catalog.GetString ("Images"), 2),
 			new TileGroupInfo (TileGroup.Audio,
 					   Catalog.GetString ("Audio"), 2),
 			new TileGroupInfo (TileGroup.Video,
@@ -39,23 +40,67 @@ namespace Search.Tiles {
 			new TileGroupInfo (TileGroup.Conversations,
 					   Catalog.GetString ("Conversations"), 5),
 			new TileGroupInfo (TileGroup.Website,
-					   Catalog.GetString ("Website"), 2),
+					   Catalog.GetString ("Websites"), 2),
 			new TileGroupInfo (TileGroup.Feed,
-					   Catalog.GetString ("News Feed"), 2),
+					   Catalog.GetString ("News Feeds"), 2),
 			new TileGroupInfo (TileGroup.Archive,
-					   Catalog.GetString ("Archive"), 2),
+					   Catalog.GetString ("Archives"), 2),
 		};
 
-		public static DateTime ParseTimestamp (string timestamp)
+		public static ScopeType TileGroupToScopeType (TileGroup group)
 		{
-			DateTime dt;
-			dt = new DateTime (Int32.Parse (timestamp.Substring (0, 4)),
-					   Int32.Parse (timestamp.Substring (4, 2)),
-					   Int32.Parse (timestamp.Substring (6, 2)),
-					   Int32.Parse (timestamp.Substring (8, 2)),
-					   Int32.Parse (timestamp.Substring (10, 2)),
-					   Int32.Parse (timestamp.Substring (12, 2)));
-			return dt;
+			switch (group) {
+
+			case TileGroup.Application:
+				return ScopeType.Applications;
+
+			case TileGroup.Calendar:
+				return ScopeType.Calendar;
+
+			case TileGroup.Contact:
+				return ScopeType.Contacts;
+
+			// This TileGroup exists but does not seem to be used
+			// case TileGroup.Folder:
+			case TileGroup.Documents:
+				return ScopeType.Documents;
+
+			case TileGroup.Conversations:
+				return ScopeType.Conversations;
+
+			case TileGroup.Image:
+				return ScopeType.Images;
+
+			case TileGroup.Audio:
+				return ScopeType.Media;
+				
+			case TileGroup.Video:
+				return ScopeType.Media;
+
+			case TileGroup.Folder:
+				return ScopeType.Folders;
+				
+			case TileGroup.Website:
+				return ScopeType.Websites;
+
+			case TileGroup.Feed:
+				return ScopeType.Feeds;
+
+			case TileGroup.Archive:
+				return ScopeType.Archives;
+				
+			}
+
+			Console.WriteLine ("Error: Could not find ScopeType for Group: {0}",group);
+			return ScopeType.Nothing;
+		}
+
+		public static string GetFirstPropertyOfParent (Beagle.Hit hit, string prop)
+		{
+			if (hit.ParentUri == null)
+				return hit.GetFirstProperty (prop);
+			else
+				return hit.GetFirstProperty ("parent:" + prop);
 		}
 
 		private static DateTimeFormatInfo DateTimeFormat = CultureInfo.CurrentCulture.DateTimeFormat;
@@ -64,12 +109,43 @@ namespace Search.Tiles {
 		private static string MonthDayPattern = DateTimeFormat.MonthDayPattern;
 		private static string LongDatePattern = DateTimeFormat.LongDatePattern.Replace ("dddd, ", "").Replace ("dddd ", "").Replace (" dddd", "");
 
+		private static string NiceDatePattern (DateTime dt, string month_day_pattern, string year_month_pattern)
+		{
+			if (dt.Year <= 1970)
+				return "-";
+
+			dt = dt.ToLocalTime ().Date;
+			DateTime today = DateTime.Today;
+			TimeSpan one_day = new TimeSpan (TimeSpan.TicksPerDay);
+
+			if (dt == today - one_day)
+				return Catalog.GetString ("Yesterday");
+			else if (dt == today)
+				return Catalog.GetString ("Today");
+			else if (dt == today + one_day)
+				return Catalog.GetString ("Tomorrow");
+
+			TimeSpan span;
+
+			if (today > dt)
+				span = today - dt;
+			else
+				span = dt - today;
+
+			if (span.TotalDays < 7)
+				return dt.ToString ("dddd"); // "Tuesday"
+			else if (dt.Year == today.Year || span.TotalDays < 180)
+				return dt.ToString (month_day_pattern);
+			else
+				return dt.ToString (year_month_pattern);
+		}
+
 		public static string NiceShortDate (string timestamp)
 		{
 			DateTime dt;
 
 			try {
-				dt = ParseTimestamp (timestamp);
+				dt = StringFu.StringToDateTime (timestamp);
 			} catch {
 				return "";
 			}
@@ -79,35 +155,8 @@ namespace Search.Tiles {
 
 		public static string NiceShortDate (DateTime dt)
 		{
-			if (dt.Year <= 1970)
-				return "-";
-
-			dt = dt.ToLocalTime ();
-			DateTime today = DateTime.Today;
-
-			TimeSpan span;
-			bool future;
-
-			if (today > dt) {
-				span = today - dt;
-				future = false;
-			} else {
-				span = dt - today;
-				future = true;
-			}
-
-			if (span.TotalDays < 1)
-				return Catalog.GetString ("Today");
-			else if (span.TotalDays < 2 && ! future)
-				return Catalog.GetString ("Yesterday");
-			else if (span.TotalDays < 2 && future)
-				return Catalog.GetString ("Tomorrow");
-			else if (span.TotalDays < 7)
-				return dt.ToString ("dddd"); // "Tuesday"
-			else if (dt.Year == today.Year || span.TotalDays < 180)
-				return dt.ToString (ShortMonthDayPattern); // "Jul 4"
-			else
-				return dt.ToString (ShortYearMonthPattern); // "Jan 2001"
+			// "Jul 4" and "Jan 2001", respectively.
+			return NiceDatePattern (dt, ShortMonthDayPattern, ShortYearMonthPattern);
 		}
 
 		public static string NiceLongDate (string timestamp)
@@ -115,7 +164,7 @@ namespace Search.Tiles {
 			DateTime dt;
 
 			try {
-				dt = ParseTimestamp (timestamp);
+				dt = StringFu.StringToDateTime (timestamp);
 			} catch {
 				return "";
 			}
@@ -125,35 +174,8 @@ namespace Search.Tiles {
 
 		public static string NiceLongDate (DateTime dt)
 		{
-			if (dt.Year <= 1970)
-				return "-";
-
-			dt = dt.ToLocalTime ();
-			DateTime today = DateTime.Today;
-
-			TimeSpan span;
-			bool future;
-
-			if (today > dt) {
-				span = today - dt;
-				future = false;
-			} else {
-				span = dt - today;
-				future = true;
-			}
-
-			if (span.TotalDays < 1)
-				return Catalog.GetString ("Today");
-			else if (span.TotalDays < 2 && ! future)
-				return Catalog.GetString ("Yesterday");
-			else if (span.TotalDays < 2 && future)
-				return Catalog.GetString ("Tomorrow");
-			else if (span.TotalDays < 7)
-				return dt.ToString ("dddd"); // "Tuesday"
-			else if (dt.Year == today.Year || span.TotalDays < 180)
-				return dt.ToString (MonthDayPattern); // "July 4"
-			else
-				return dt.ToString (LongDatePattern); // January 7, 2001
+			// "July 4" and "January 7, 2001", respectively.
+			return NiceDatePattern (dt, MonthDayPattern, LongDatePattern);
 		}
 
 		public static string NiceVeryLongDate (string timestamp)
@@ -161,7 +183,7 @@ namespace Search.Tiles {
 			DateTime dt;
 
 			try {
-				dt = ParseTimestamp (timestamp);
+				dt = StringFu.StringToDateTime (timestamp);
 			} catch {
 				return "";
 			}
@@ -174,8 +196,16 @@ namespace Search.Tiles {
 			if (dt.Year <= 1970)
 				return "-";
 
-			dt = dt.ToLocalTime ();
+			dt = dt.ToLocalTime ().Date;
 			DateTime today = DateTime.Today;
+			TimeSpan one_day = new TimeSpan (TimeSpan.TicksPerDay);
+
+			if (dt == today - one_day)
+				return Catalog.GetString ("Yesterday");
+			else if (dt == today)
+				return Catalog.GetString ("Today");
+			else if (dt == today + one_day)
+				return Catalog.GetString ("Tomorrow");
 
 			TimeSpan span;
 			bool future;
@@ -188,13 +218,7 @@ namespace Search.Tiles {
 				future = true;
 			}
 
-			if (span.TotalDays < 1)
-				return Catalog.GetString ("Today");
-			else if (span.TotalDays < 2 && ! future)
-				return Catalog.GetString ("Yesterday");
-			else if (span.TotalDays < 2 && future)
-				return Catalog.GetString ("Tomorrow");
-			else if (span.TotalDays < 7)
+			if (span.TotalDays < 7)
 				return dt.ToString ("dddd"); // "Tuesday"
 			else if (span.TotalDays < 30 && ! future)
 				return String.Format (Catalog.GetPluralString ("{0} week ago", "{0} weeks ago", span.Days / 7) + " ({1:MMMM d, yyyy})", span.Days / 7, dt);
